@@ -54,16 +54,18 @@ enum
   PROP_COLOR_FOREGROUND,
   PROP_COLOR_BACKGROUND,
   PROP_COLOR_BACKGROUND_VARY,
-  PROP_COLOR_CURSOR_FG,
+  PROP_COLOR_CURSOR_FOREGROUND,
   PROP_COLOR_CURSOR,
   PROP_COLOR_CURSOR_USE_DEFAULT,
   PROP_COLOR_SELECTION,
-  PROP_COLOR_SELECTION_BG,
+  PROP_COLOR_SELECTION_BACKGROUND,
   PROP_COLOR_SELECTION_USE_DEFAULT,
   PROP_COLOR_BOLD,
   PROP_COLOR_BOLD_USE_DEFAULT,
   PROP_COLOR_PALETTE,
+  PROP_COLOR_BOLD_IS_BRIGHT,
   PROP_COMMAND_LOGIN_SHELL,
+  PROP_COMMAND_UPDATE_RECORDS,
   PROP_RUN_CUSTOM_COMMAND,
   PROP_CUSTOM_COMMAND,
   PROP_DROPDOWN_ANIMATION_TIME,
@@ -75,6 +77,7 @@ enum
   PROP_DROPDOWN_HEIGHT,
   PROP_DROPDOWN_OPACITY,
   PROP_DROPDOWN_POSITION,
+  PROP_DROPDOWN_POSITION_VERTICAL,
   PROP_DROPDOWN_MOVE_TO_ACTIVE,
   PROP_DROPDOWN_ALWAYS_SHOW_TABS,
   PROP_DROPDOWN_SHOW_BORDERS,
@@ -102,10 +105,12 @@ enum
   PROP_MISC_HIGHLIGHT_URLS,
   PROP_MISC_MIDDLE_CLICK_OPENS_URI,
   PROP_MISC_COPY_ON_SELECT,
-  PROP_MISC_DEFAULT_WORKING_DIR,
+  PROP_USE_DEFAULT_WORKING_DIR,
+  PROP_DEFAULT_WORKING_DIR,
   PROP_MISC_REWRAP_ON_RESIZE,
-  PROP_MISC_SHIFT_ARROWS_SCROLL,
+  PROP_MISC_USE_SHIFT_ARROWS_TO_SCROLL,
   PROP_MISC_SLIM_TABS,
+  PROP_MISC_NEW_TAB_ADJACENT,
   PROP_SCROLLING_BAR,
   PROP_SCROLLING_LINES,
   PROP_SCROLLING_ON_OUTPUT,
@@ -119,6 +124,9 @@ enum
   PROP_WORD_CHARS,
   PROP_TAB_ACTIVITY_COLOR,
   PROP_TAB_ACTIVITY_TIMEOUT,
+  PROP_TEXT_BLINK_MODE,
+  PROP_CELL_WIDTH_SCALE,
+  PROP_CELL_HEIGHT_SCALE,
   N_PROPERTIES,
 };
 
@@ -263,6 +271,18 @@ static void
 terminal_preferences_class_init (TerminalPreferencesClass *klass)
 {
   GObjectClass *gobject_class;
+  guint         i;
+  const GType   enum_types[] = {
+    GTK_TYPE_POSITION_TYPE,
+    TERMINAL_TYPE_BACKGROUND_STYLE,
+    TERMINAL_TYPE_BACKGROUND,
+    TERMINAL_TYPE_SCROLLBAR,
+    TERMINAL_TYPE_TITLE,
+    TERMINAL_TYPE_ERASE_BINDING,
+    TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING,
+    TERMINAL_TYPE_CURSOR_SHAPE,
+    TERMINAL_TYPE_TEXT_BLINK_MODE
+  };
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = terminal_preferences_dispose;
@@ -281,22 +301,10 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
     g_value_register_transform_func (G_TYPE_STRING, G_TYPE_DOUBLE, transform_string_to_double);
   if (!g_value_type_transformable (G_TYPE_STRING, G_TYPE_UINT))
     g_value_register_transform_func (G_TYPE_STRING, G_TYPE_UINT, transform_string_to_uint);
-  if (!g_value_type_transformable (G_TYPE_STRING, GTK_TYPE_POSITION_TYPE))
-    g_value_register_transform_func (G_TYPE_STRING, GTK_TYPE_POSITION_TYPE, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_BACKGROUND_STYLE))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_BACKGROUND_STYLE, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_BACKGROUND))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_BACKGROUND, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_SCROLLBAR))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_SCROLLBAR, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_TITLE))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_TITLE, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_ERASE_BINDING))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_ERASE_BINDING, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING, transform_string_to_enum);
-  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_CURSOR_SHAPE))
-    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_CURSOR_SHAPE, transform_string_to_enum);
+  /* enum types */
+  for (i = 0; i < G_N_ELEMENTS (enum_types); ++i)
+    if (!g_value_type_transformable (G_TYPE_STRING, enum_types[i]))
+      g_value_register_transform_func (G_TYPE_STRING, enum_types[i], transform_string_to_enum);
 
   /**
    * TerminalPreferences:background-mode:
@@ -419,10 +427,10 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * TerminalPreferences:color-cursor-fg:
+   * TerminalPreferences:color-cursor-foreground:
    **/
-  preferences_props[PROP_COLOR_CURSOR_FG] =
-      g_param_spec_string ("color-cursor-fg",
+  preferences_props[PROP_COLOR_CURSOR_FOREGROUND] =
+      g_param_spec_string ("color-cursor-foreground",
                            NULL,
                            "ColorCursorForeground",
                            "",
@@ -459,10 +467,10 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * TerminalPreferences:color-selection-bg:
+   * TerminalPreferences:color-selection-background:
    **/
-  preferences_props[PROP_COLOR_SELECTION_BG] =
-      g_param_spec_string ("color-selection-bg",
+  preferences_props[PROP_COLOR_SELECTION_BACKGROUND] =
+      g_param_spec_string ("color-selection-background",
                            NULL,
                            "ColorSelectionBackground",
                            "",
@@ -523,6 +531,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            "#ffffff",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:color-bold-is-bright:
+   **/
+  preferences_props[PROP_COLOR_BOLD_IS_BRIGHT] =
+      g_param_spec_boolean ("color-bold-is-bright",
+                            NULL,
+                            "ColorBoldIsBright",
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:tab-activity-color:
    **/
@@ -553,6 +571,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:command-update-records:
+   **/
+  preferences_props[PROP_COMMAND_UPDATE_RECORDS] =
+      g_param_spec_boolean ("command-update-records",
+                            NULL,
+                            "CommandUpdateRecords",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:run-custom-command:
    **/
@@ -573,6 +601,26 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            "",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:use-default-working-dir:
+   **/
+  preferences_props[PROP_USE_DEFAULT_WORKING_DIR] =
+      g_param_spec_boolean ("use-default-working-dir",
+                            NULL,
+                            "UseDefaultWorkingDir",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:default-working-dir:
+   **/
+  preferences_props[PROP_DEFAULT_WORKING_DIR] =
+      g_param_spec_string ("default-working-dir",
+                           NULL,
+                           "DefaultWorkingDir",
+                           "",
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:dropdown-animation-time:
    **/
@@ -666,6 +714,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                          0, 100, 50,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:dropdown-position-vertical:
+   **/
+  preferences_props[PROP_DROPDOWN_POSITION_VERTICAL] =
+      g_param_spec_uint ("dropdown-position-vertical",
+                         NULL,
+                         "DropdownPositionVertical",
+                         0, 100, 0,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:move-to-active:
    **/
@@ -938,16 +996,6 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  /**
-   * TerminalPreferences:misc-default-working-dir:
-   **/
-  preferences_props[PROP_MISC_DEFAULT_WORKING_DIR] =
-      g_param_spec_string ("misc-default-working-dir",
-                           NULL,
-                           "MiscDefaultWorkingDir",
-                           "",
-                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   /**
    * TerminalPreferences:misc-rewrap-on-resize:
    **/
@@ -959,17 +1007,17 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * TerminalPreferences:misc-shift-arrows-scroll:
+   * TerminalPreferences:misc-use-shift-arrows-to-scroll:
    **/
-  preferences_props[PROP_MISC_SHIFT_ARROWS_SCROLL] =
-      g_param_spec_boolean ("misc-shift-arrows-scroll",
+  preferences_props[PROP_MISC_USE_SHIFT_ARROWS_TO_SCROLL] =
+      g_param_spec_boolean ("misc-use-shift-arrows-to-scroll",
                             NULL,
                             "MiscUseShiftArrowsToScroll",
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * TerminalPreferences:misc-shift-arrows-scroll:
+   * TerminalPreferences:misc-slim-tabs:
    **/
   preferences_props[PROP_MISC_SLIM_TABS] =
       g_param_spec_boolean ("misc-slim-tabs",
@@ -978,6 +1026,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-new-tab-adjacent:
+   **/
+  preferences_props[PROP_MISC_NEW_TAB_ADJACENT] =
+      g_param_spec_boolean ("misc-new-tab-adjacent",
+                            NULL,
+                            "MiscNewTabAdjacent",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:scrolling-bar:
    **/
@@ -1094,6 +1152,37 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            "-A-Za-z0-9,./?%&#:_=+@~",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:text-blink-mode:
+   **/
+  preferences_props[PROP_TEXT_BLINK_MODE] =
+      g_param_spec_enum ("text-blink-mode",
+                         NULL,
+                         "TextBlinkMode",
+                         TERMINAL_TYPE_TEXT_BLINK_MODE,
+                         TERMINAL_TEXT_BLINK_MODE_ALWAYS,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:cell-width-scale:
+   **/
+  preferences_props[PROP_CELL_WIDTH_SCALE] =
+      g_param_spec_double ("cell-width-scale",
+                           NULL,
+                           "CellWidthScale",
+                           1.0, 2.0, 1.0,
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:cell-height-scale:
+   **/
+  preferences_props[PROP_CELL_HEIGHT_SCALE] =
+      g_param_spec_double ("cell-height-scale",
+                           NULL,
+                           "CellHeightScale",
+                           1.0, 2.0, 1.0,
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /* install all properties */
   g_object_class_install_properties (gobject_class, N_PROPERTIES, preferences_props);
 }
